make uci_lookup_ptr() use the extended flag properly again
[project/uci.git] / list.c
diff --git a/list.c b/list.c
index fb9d38a..c683dd2 100644 (file)
--- a/list.c
+++ b/list.c
@@ -36,7 +36,7 @@ static inline void uci_list_fixup(struct uci_list *ptr)
  * uci_alloc_generic allocates a new uci_element with payload
  * payload is appended to the struct to save memory and reduce fragmentation
  */
-static struct uci_element *
+__private struct uci_element *
 uci_alloc_generic(struct uci_context *ctx, int type, const char *name, int size)
 {
        struct uci_element *e;
@@ -62,7 +62,7 @@ done:
        return e;
 }
 
-static void
+__private void
 uci_free_element(struct uci_element *e)
 {
        if (e->name)
@@ -127,6 +127,22 @@ uci_alloc_list(struct uci_section *s, const char *name)
        return o;
 }
 
+/* Based on an efficient hash function published by D. J. Bernstein */
+static unsigned int djbhash(unsigned int hash, char *str)
+{
+       int len = strlen(str);
+       int i;
+
+       /* initial value */
+       if (hash == ~0)
+               hash = 5381;
+
+       for(i = 0; i < len; i++) {
+               hash = ((hash << 5) + hash) + str[i];
+       }
+       return (hash & 0x7FFFFFFF);
+}
+
 /* fix up an unnamed section, e.g. after adding options to it */
 __private void uci_fixup_section(struct uci_context *ctx, struct uci_section *s)
 {
@@ -208,8 +224,8 @@ uci_alloc_package(struct uci_context *ctx, const char *name)
        p = uci_alloc_element(ctx, package, name, 0);
        p->ctx = ctx;
        uci_list_init(&p->sections);
-       uci_list_init(&p->history);
-       uci_list_init(&p->saved_history);
+       uci_list_init(&p->delta);
+       uci_list_init(&p->saved_delta);
        return p;
 }
 
@@ -227,11 +243,11 @@ uci_free_package(struct uci_package **package)
        uci_foreach_element_safe(&p->sections, tmp, e) {
                uci_free_section(uci_to_section(e));
        }
-       uci_foreach_element_safe(&p->history, tmp, e) {
-               uci_free_history(uci_to_history(e));
+       uci_foreach_element_safe(&p->delta, tmp, e) {
+               uci_free_delta(uci_to_delta(e));
        }
-       uci_foreach_element_safe(&p->saved_history, tmp, e) {
-               uci_free_history(uci_to_history(e));
+       uci_foreach_element_safe(&p->saved_delta, tmp, e) {
+               uci_free_delta(uci_to_delta(e));
        }
        uci_free_element(&p->e);
        *package = NULL;
@@ -371,10 +387,14 @@ uci_lookup_ptr(struct uci_context *ctx, struct uci_ptr *ptr, char *str, bool ext
 
        /* if the section name validates as a regular name, pass through
         * to the regular uci_lookup function call */
-       if (ptr->flags & UCI_LOOKUP_EXTENDED)
-               e = uci_lookup_ext_section(ctx, ptr);
-       else
+       if (ptr->flags & UCI_LOOKUP_EXTENDED) {
+               if (extended)
+                       e = uci_lookup_ext_section(ctx, ptr);
+               else
+                       UCI_THROW(ctx, UCI_ERR_INVAL);
+       } else {
                e = uci_lookup_list(&ptr->p->sections, ptr->section);
+       }
 
        if (!e)
                goto abort;
@@ -401,8 +421,8 @@ notfound:
        return 0;
 }
 
-static struct uci_element *
-expand_ptr(struct uci_context *ctx, struct uci_ptr *ptr, bool complete)
+__private struct uci_element *
+uci_expand_ptr(struct uci_context *ctx, struct uci_ptr *ptr, bool complete)
 {
        UCI_ASSERT(ctx, ptr != NULL);
 
@@ -436,8 +456,8 @@ static void uci_add_element_list(struct uci_context *ctx, struct uci_ptr *ptr, b
        struct uci_package *p;
 
        p = ptr->p;
-       if (!internal && p->has_history)
-               uci_add_history(ctx, &p->history, UCI_CMD_LIST_ADD, ptr->section, ptr->option, ptr->value);
+       if (!internal && p->has_delta)
+               uci_add_delta(ctx, &p->delta, UCI_CMD_LIST_ADD, ptr->section, ptr->option, ptr->value);
 
        e = uci_alloc_generic(ctx, UCI_TYPE_ITEM, ptr->value, sizeof(struct uci_option));
        uci_list_add(&ptr->o->v.list, &e->list);
@@ -445,7 +465,7 @@ static void uci_add_element_list(struct uci_context *ctx, struct uci_ptr *ptr, b
 
 int uci_rename(struct uci_context *ctx, struct uci_ptr *ptr)
 {
/* NB: UCI_INTERNAL use means without history tracking */
        bool internal = ctx->internal;
        struct uci_element *e;
        struct uci_package *p;
@@ -453,14 +473,14 @@ int uci_rename(struct uci_context *ctx, struct uci_ptr *ptr)
 
        UCI_HANDLE_ERR(ctx);
 
-       e = expand_ptr(ctx, ptr, true);
+       e = uci_expand_ptr(ctx, ptr, true);
        p = ptr->p;
 
        UCI_ASSERT(ctx, ptr->s);
        UCI_ASSERT(ctx, ptr->value);
 
-       if (!internal && p->has_history)
-               uci_add_history(ctx, &p->history, UCI_CMD_RENAME, ptr->section, ptr->option, ptr->value);
+       if (!internal && p->has_delta)
+               uci_add_delta(ctx, &p->delta, UCI_CMD_RENAME, ptr->section, ptr->option, ptr->value);
 
        n = uci_strdup(ctx, ptr->value);
        if (e->name)
@@ -481,9 +501,9 @@ int uci_reorder_section(struct uci_context *ctx, struct uci_section *s, int pos)
        UCI_HANDLE_ERR(ctx);
 
        uci_list_set_pos(&s->package->sections, &s->e.list, pos);
-       if (!ctx->internal && p->has_history) {
+       if (!ctx->internal && p->has_delta) {
                sprintf(order, "%d", pos);
-               uci_add_history(ctx, &p->history, UCI_CMD_REORDER, s->e.name, NULL, order);
+               uci_add_delta(ctx, &p->delta, UCI_CMD_REORDER, s->e.name, NULL, order);
        }
 
        return 0;
@@ -499,8 +519,8 @@ int uci_add_section(struct uci_context *ctx, struct uci_package *p, const char *
        s = uci_alloc_section(p, type, NULL);
        uci_fixup_section(ctx, s);
        *res = s;
-       if (!internal && p->has_history)
-               uci_add_history(ctx, &p->history, UCI_CMD_ADD, s->e.name, NULL, type);
+       if (!internal && p->has_delta)
+               uci_add_delta(ctx, &p->delta, UCI_CMD_ADD, s->e.name, NULL, type);
 
        return 0;
 }
@@ -514,13 +534,13 @@ int uci_delete(struct uci_context *ctx, struct uci_ptr *ptr)
 
        UCI_HANDLE_ERR(ctx);
 
-       e = expand_ptr(ctx, ptr, true);
+       e = uci_expand_ptr(ctx, ptr, true);
        p = ptr->p;
 
        UCI_ASSERT(ctx, ptr->s);
 
-       if (!internal && p->has_history)
-               uci_add_history(ctx, &p->history, UCI_CMD_REMOVE, ptr->section, ptr->option, NULL);
+       if (!internal && p->has_delta)
+               uci_add_delta(ctx, &p->delta, UCI_CMD_REMOVE, ptr->section, ptr->option, NULL);
 
        uci_free_any(&e);
 
@@ -534,14 +554,14 @@ int uci_delete(struct uci_context *ctx, struct uci_ptr *ptr)
 
 int uci_add_list(struct uci_context *ctx, struct uci_ptr *ptr)
 {
/* NB: UCI_INTERNAL use means without history tracking */
        bool internal = ctx->internal;
        struct uci_option *prev = NULL;
        const char *value2 = NULL;
 
        UCI_HANDLE_ERR(ctx);
 
-       expand_ptr(ctx, ptr, false);
+       uci_expand_ptr(ctx, ptr, false);
        UCI_ASSERT(ctx, ptr->s);
        UCI_ASSERT(ctx, ptr->value);
 
@@ -575,11 +595,11 @@ int uci_add_list(struct uci_context *ctx, struct uci_ptr *ptr)
 
 int uci_set(struct uci_context *ctx, struct uci_ptr *ptr)
 {
/* NB: UCI_INTERNAL use means without history tracking */
        bool internal = ctx->internal;
 
        UCI_HANDLE_ERR(ctx);
-       expand_ptr(ctx, ptr, false);
+       uci_expand_ptr(ctx, ptr, false);
        UCI_ASSERT(ctx, ptr->value);
        UCI_ASSERT(ctx, ptr->s || (!ptr->option && ptr->section));
        if (!ptr->option && ptr->value[0]) {
@@ -628,8 +648,8 @@ int uci_set(struct uci_context *ctx, struct uci_ptr *ptr)
                UCI_THROW(ctx, UCI_ERR_INVAL);
        }
 
-       if (!internal && ptr->p->has_history)
-               uci_add_history(ctx, &ptr->p->history, UCI_CMD_CHANGE, ptr->section, ptr->option, ptr->value);
+       if (!internal && ptr->p->has_delta)
+               uci_add_delta(ctx, &ptr->p->delta, UCI_CMD_CHANGE, ptr->section, ptr->option, ptr->value);
 
        return 0;
 }
@@ -445,7 +465,7 @@ static void uci_add_element_list(struct uci_context *ctx, struct uci_ptr *ptr, b
 int uci_rename(struct uci_context *ctx, struct uci_ptr *ptr)
 {
-       /* NB: UCI_INTERNAL use means without history tracking */
+       /* NB: UCI_INTERNAL use means without delta tracking */
        bool internal = ctx->internal;
        struct uci_element *e;
@@ -534,14 +554,14 @@ int uci_delete(struct uci_context *ctx, struct uci_ptr *ptr)
 int uci_add_list(struct uci_context *ctx, struct uci_ptr *ptr)
 {
-       /* NB: UCI_INTERNAL use means without history tracking */
+       /* NB: UCI_INTERNAL use means without delta tracking */
        bool internal = ctx->internal;
        struct uci_option *prev = NULL;
@@ -575,11 +595,11 @@ int uci_add_list(struct uci_context *ctx, struct uci_ptr *ptr)
 int uci_set(struct uci_context *ctx, struct uci_ptr *ptr)
 {
-       /* NB: UCI_INTERNAL use means without history tracking */
+       /* NB: UCI_INTERNAL use means without delta tracking */
        bool internal = ctx->internal;
 